
#include <string>
#include <list>
#include <typeinfo>

#include "coast.hpp"
#include "c_code_provider.hpp"

using namespace std;





/*
    CCodeProvider

    C language code provider.
    (Gabriele Budelacci, 2003-02-08)
*/



// compile_assembly:
//	Compile assembly fom the specified source.
bool CCodeProvider::compile_assembly( CoastElement *ast )
{
	return true;
}//compile_assembly





// generate_code:
//	Generate the source code from ast.
string CCodeProvider::generate_code( CoastElement* ast )
{
	string assembly = "";
	assembly += "//------------------------------------------------------------------------------\n";
	assembly += "// <autogenerated>\n";
	assembly += "//     This code was generated by a tool.\n";
	assembly += "//     coast version: " + (string) COAST_VERSION + "\n";
	assembly += "//\n";
	assembly += "//     Changes to this file may cause incorrect behavior and will be lost if\n";
	assembly += "//     the code is regenerated.\n";
	assembly += "// </autogenerated>\n";
	assembly += "//------------------------------------------------------------------------------\n\n";

	return assembly + get_assembly( ast );
}//generate_code





// get_assembly:
//	Get the assembly code of a element.
string CCodeProvider::get_assembly( CoastApplication*                   application )
{
	string assembly = "";

	CoastMemberCollection::iterator it;
	for ( it = application->members->begin() ; it != application->members->end() ; it++ )
	{
		// Visit a list from begin to end is reverse order.
		// We need to re-reverse the results.
//		string tmp_asm = (*it)->get_assembly();
		string tmp_asm = get_assembly( *it );
		tmp_asm += "\n\n\n";
		assembly = tmp_asm + assembly;
		// original code was:
//			assembly += (*it)->get_assembly();
		//	assembly += get_assembly( *it );
		//	assembly += "\n\n\n";
	}

	// paddle return value...
	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastArgumentDeclarationExpression* expression  )
{
	return get_assembly( expression->type ) + " " + expression->name;
}//get_assembly



string CCodeProvider::get_assembly( CoastArgumentReferenceExpression*   expression  )
{
	string assembly = expression->name;

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastArrayIndexerExpression*        expression  )
{
	// validating object
	if ( expression->target == NULL )
		return "CoastArrayIndexerExpression: NO TARGET EXPRESSION";
	if ( expression->indices->begin() == expression->indices->end() )
		return "CoastArrayIndexerExpression: NO INDEX EXPRESSION";

	string assembly = get_assembly( expression->target );
	assembly += "[ ";

	// indexes
	CoastElementCollection::iterator it = expression->indices->begin();
	string index_asm = get_assembly( *it );

	// get other optional indexes assembly.
	while ( ++it != expression->indices->end() )
		index_asm = get_assembly( *it ) + ", " + index_asm;

	assembly += index_asm + " ]";

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastAssignStatement*               statement   )
{
	string assembly = "";

//	assembly += statement->lvalue->get_assembly();
	assembly += get_assembly( statement->lvalue );
	assembly += " = ";
//	assembly += statement->rvalue->get_assembly();
	assembly += get_assembly( statement->rvalue );
	assembly += ";";

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastBinaryOperatorExpression*      expression  )
{
	string assembly = "";

	assembly += "(" + get_assembly( expression->lvalue );

	switch ( expression->op )
	{
		case Add:
			assembly += "+";
			break;
		case Assign:
			assembly += "=";
			break;
		case BitwiseAnd:
			assembly += "&";
			break;
		case BitwiseExor:
			assembly += "^";
			break;
		case BitwiseOr:
			assembly += "|";
			break;
		case BooleanAnd:
			assembly += "&&";
			break;
		case BooleanOr:
			assembly += "||";
			break;
		case Divide:
			assembly += "/";
			break;
		case GreaterThan:
			assembly += ">";
			break;
		case GreaterThanOrEqual:
			assembly += ">=";
			break;
		case IdentityEquality:
			assembly += "==";
			break;
		case IdentityInequality:
			assembly += "!=";
			break;
		case LessThan:
			assembly += "<";
			break;
		case LessThanOrEqual:
			assembly += "<=";
			break;
		case Modulus:
			assembly += "%";
			break;
		case Multiply:
			assembly += "*";
			break;
		case ShiftLeft:
			assembly += "<<";
			break;
		case ShiftRight:
			assembly += ">>";
			break;
		case Subtract:
			assembly += "-";
			break;
		default:
			assembly += "CoastBinaryOperatorType UNMANAGED";
	}

	assembly += get_assembly( expression->rvalue ) + ")";

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastBreakStatement*                statement   )
{
	return "break;";
}//get_assembly



string CCodeProvider::get_assembly( CoastCommentStatement*              statement   )
{
	string assembly;

	assembly = "/* " + statement->comment + " */";

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastConditionStatement*            statement   )
{
	string assembly = "";

	if ( statement->condition == NULL )
		return "CoastConditionStatement: NO CONDITION SPECIFIED";

	assembly += "if ( ";
	assembly += get_assembly( statement->condition );
	assembly += " )\n";
	assembly += "{\n";

	// this is needed to manage correctly the statements assembly
	string statements_asm = "";

	// true statements
	CoastStatementCollection::iterator it;
	for ( it = statement->true_statements->begin() ; it != statement->true_statements->end() ; it++ )
	{
		// We must reverse the assembly string.
		// See get_assembly( CoastApplication *& ).
		string tmp_asm = "    ";
		tmp_asm += get_assembly( *it );
		tmp_asm += "\n";
		statements_asm = tmp_asm + statements_asm;
	}

	assembly += statements_asm;
	assembly += "}\n";


	// check for optional false statements...
	if ( statement->false_statements->begin() != statement->false_statements->end() )
	{
		assembly += "else";
		assembly += "{\n";

		// this is needed to manage correctly the statements assembly
		string statements_asm = "";

		// true statements
		CoastStatementCollection::iterator it;
		for ( it = statement->false_statements->begin() ; it != statement->false_statements->end() ; it++ )
		{
			// We must reverse the assembly string.
			// See get_assembly( CoastApplication *& ).
			string tmp_asm = "    ";
			tmp_asm += get_assembly( *it );
			tmp_asm += "\n";
			statements_asm = tmp_asm + statements_asm;
		}

		assembly += statements_asm;
		assembly += "}\n";
	}

	assembly += "}\n";

    return assembly;

}//get_assembly



string CCodeProvider::get_assembly( CoastContinueStatement*             statement   )
{
	return "continue;";
}//get_assembly



string CCodeProvider::get_assembly( CoastEntryPointFunction*            function    )
{
	string assembly = "";

	assembly += "int " + function->name + "( int argc, char* argv[] )\n";
	assembly += "{\n";

	// this is needed to manage correctly the statements assembly
	string statements_asm = "";

	CoastStatementCollection::iterator it;
	for ( it = function->statements->begin() ; it != function->statements->end() ; it++ )
	{
		// We need to reverse the assembly string.
		// See get_assembly( CoastApplication *& ).
		string tmp_asm = "    ";
		tmp_asm += get_assembly( *it );
		tmp_asm += "\n";
		statements_asm = tmp_asm + statements_asm;
	}

	assembly += statements_asm;

	assembly += "    return 0;\n";
	assembly += "}\n";

    return assembly;

}//get_assembly



string CCodeProvider::get_assembly( CoastElement*                       element     )
{
	if ( typeid( *element ) == typeid( CoastApplication ) )
		return get_assembly( dynamic_cast<CoastApplication*>( element ) );

	if ( typeid( *element ) == typeid( CoastArgumentDeclarationExpression ) )
		return get_assembly( dynamic_cast<CoastArgumentDeclarationExpression*>( element ) );

	if ( typeid( *element ) == typeid( CoastArgumentReferenceExpression ) )
		return get_assembly( dynamic_cast<CoastArgumentReferenceExpression*>( element ) );

	if ( typeid( *element ) == typeid( CoastArrayIndexerExpression ) )
		return get_assembly( dynamic_cast<CoastArrayIndexerExpression*>( element ) );

	if ( typeid( *element ) == typeid( CoastAssignStatement ) )
		return get_assembly( dynamic_cast<CoastAssignStatement*>( element ) );

	if ( typeid( *element ) == typeid( CoastBinaryOperatorExpression ) )
		return get_assembly( dynamic_cast<CoastBinaryOperatorExpression*>( element ) );

	if ( typeid( *element ) == typeid( CoastBreakStatement ) )
		return get_assembly( dynamic_cast<CoastBreakStatement*>( element ) );

	if ( typeid( *element ) == typeid( CoastCommentStatement ) )
		return get_assembly( dynamic_cast<CoastCommentStatement*>( element ) );

	if ( typeid( *element ) == typeid( CoastConditionStatement ) )
		return get_assembly( dynamic_cast<CoastConditionStatement*>( element ) );

	if ( typeid( *element ) == typeid( CoastContinueStatement ) )
		return get_assembly( dynamic_cast<CoastContinueStatement*>( element ) );

	if ( typeid( *element ) == typeid( CoastEntryPointFunction ) )
		return get_assembly( dynamic_cast<CoastEntryPointFunction*>( element ) );

	if ( typeid( *element ) == typeid( CoastExpressionStatement ) )
		return get_assembly( dynamic_cast<CoastExpressionStatement*>( element ) );

	if ( typeid( *element ) == typeid( CoastFunction ) )
		return get_assembly( dynamic_cast<CoastFunction*>( element ) );

	if ( typeid( *element ) == typeid( CoastFunctionInvokeExpression ) )
		return get_assembly( dynamic_cast<CoastFunctionInvokeExpression*>( element ) );

	if ( typeid( *element ) == typeid( CoastIterationStatement ) )
		return get_assembly( dynamic_cast<CoastIterationStatement*>( element ) );

	if ( typeid( *element ) == typeid( CoastPrimitiveExpression ) )
		return get_assembly( dynamic_cast<CoastPrimitiveExpression*>( element ) );

	if ( typeid( *element ) == typeid( CoastReturnStatement ) )
		return get_assembly( dynamic_cast<CoastReturnStatement*>( element ) );

	if ( typeid( *element ) == typeid( CoastSnippetExpression ) )
		return get_assembly( dynamic_cast<CoastSnippetExpression*>( element ) );

	if ( typeid( *element ) == typeid( CoastSnippetMember ) )
		return get_assembly( dynamic_cast<CoastSnippetMember*>( element ) );

	if ( typeid( *element ) == typeid( CoastSnippetStatement ) )
		return get_assembly( dynamic_cast<CoastSnippetStatement*>( element ) );

	if ( typeid( *element ) == typeid( CoastSwitchCaseOption ) )
		return get_assembly( dynamic_cast<CoastSwitchCaseOption*>( element ) );

	if ( typeid( *element ) == typeid( CoastSwitchDefaultOption ) )
		return get_assembly( dynamic_cast<CoastSwitchDefaultOption*>( element ) );

	if ( typeid( *element ) == typeid( CoastSwitchStatement ) )
		return get_assembly( dynamic_cast<CoastSwitchStatement*>( element ) );

	if ( typeid( *element ) == typeid( CoastTypeReference ) )
		return get_assembly( dynamic_cast<CoastTypeReference*>( element ) );

	if ( typeid( *element ) == typeid( CoastUnaryOperatorExpression ) )
		return get_assembly( dynamic_cast<CoastUnaryOperatorExpression*>( element ) );

	if ( typeid( *element ) == typeid( CoastVariableDeclarationStatement ) )
		return get_assembly( dynamic_cast<CoastVariableDeclarationStatement*>( element ) );

	if ( typeid( *element ) == typeid( CoastVariableReferenceExpression ) )
		return get_assembly( dynamic_cast<CoastVariableReferenceExpression*>( element ) );

	return "get_assembly: SIMPLE COAST ELEMENT " + (string)(typeid( *element ).name());
}//get_assembly



string CCodeProvider::get_assembly( CoastExpressionStatement*           statement   )
{
	string assembly;

//	assembly = statement->expression->get_assembly() + ";";
	assembly = get_assembly( statement->expression ) + ";";

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastFunction*                      function    )
{
	string assembly = "";

	// return type
	if ( function->return_type == NULL )
		assembly += "void";
	else
		assembly += get_assembly( function->return_type );

	// name
	assembly += " " + function->name + "(";
	
	// arguments
	if ( function->arguments->begin() != function->arguments->end() )
	{
		CoastArgumentDeclarationExpressionCollection::iterator it = function->arguments->begin();

		// get first argument assembly
		string arguments_asm = get_assembly( *it );

		// get other arguments assembly.
		while ( ++it != function->arguments->end() )
			arguments_asm = get_assembly( *it ) + ", " + arguments_asm;

		assembly += " " + arguments_asm + " ";
	}

	assembly += ")\n";
	assembly += "{\n";

	// this is needed to manage correctly the statements assembly
	string statements_asm = "";

	// function statements
	CoastStatementCollection::iterator it;
	for ( it = function->statements->begin() ; it != function->statements->end() ; it++ )
	{
		// We must reverse the assembly string.
		// See get_assembly( CoastApplication *& ).
		string tmp_asm = "    ";
		tmp_asm += get_assembly( *it );
		tmp_asm += "\n";
		statements_asm = tmp_asm + statements_asm;
	}

	assembly += statements_asm;
	assembly += "}\n";

    return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastFunctionInvokeExpression*      expression  )
{
	string assembly = "";

	// name
	assembly += expression->name + "(";
	
	// arguments
	if ( expression->arguments->begin() != expression->arguments->end() )
	{
		CoastExpressionCollection::iterator it = expression->arguments->begin();

		// get first argument assembly.
		string arguments_asm = get_assembly( *it );

		// get other arguments assembly.
		while ( ++it != expression->arguments->end() )
			arguments_asm = get_assembly( *it ) + ", " + arguments_asm;

		assembly += " " + arguments_asm + " ";
	}

	assembly += ")";

    return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastIterationStatement*            statement   )
{
	string assembly = "";

	if ( statement->condition == NULL )
		return "CoastIterationStatement: NO CONDITION SPECIFIED";

	assembly += "for ( ";

	// init statement
	if ( statement->init != NULL )
		assembly += get_assembly( statement->init );
	else
		assembly += ";";
	assembly += " ";

	// condition
	assembly += get_assembly( statement->condition );

	assembly += "; )\n";

	// loop statements
	assembly += "{";

	// this is needed to manage correctly the statements assembly
	string loop_asm = "";

	// loop statements
	CoastStatementCollection::iterator it;
	for ( it = statement->statements->begin() ; it != statement->statements->end() ; it++ )
	{
		// We must reverse the assembly string.
		// See CoastApplication::get_assembly.
		string tmp_asm = "    ";
		tmp_asm += get_assembly( *it );
		tmp_asm += "\n";
		loop_asm = tmp_asm + loop_asm;
	}

	assembly += loop_asm;

	// increment statement
	if ( statement->increment != NULL )
		assembly += "    " + get_assembly( statement->increment ) + "\n";

	assembly += "}";

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastPrimitiveExpression*           expression  )
{
	string assembly;

	switch ( expression->data_type )
	{
		case 0:
			assembly = "void";
			break;
		case 1:
			assembly = expression->boolean;
			break;
		case 2:
			assembly = expression->character;
			break;
		case 3:
			assembly = expression->short_integer;
			break;
		case 4:
			assembly = expression->integer;
			break;
		case 5:
			assembly = expression->long_integer;
			break;
		case 6:
			assembly = expression->single_precision;
			break;
		case 7:
			assembly = expression->double_precision;
			break;
		case 8:
			assembly = expression->strn;
			break;
		default:
			assembly = "CoastPrimitiveExpression: Unsupported Primitive";
	}

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastReturnStatement*               statement   )
{
	string assembly = "return";

	if ( statement->expression != NULL )
	{
		assembly += " ";
		assembly += get_assembly( statement->expression );
	}
	assembly += ";";

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastSnippetExpression*             expression  )
{
	return expression->expression;
}//get_assembly



string CCodeProvider::get_assembly( CoastSnippetMember*                 member      )
{
    return member->member;
}//get_assembly



string CCodeProvider::get_assembly( CoastSnippetStatement*              statement   )
{
	return statement->statement;
}//get_assembly



string CCodeProvider::get_assembly( CoastSwitchCaseOption*              option      )
{
	string assembly = "";

	assembly += "case " + get_assembly( option->value ) + ":\n";

	// optional statements
	if ( option->statements->begin() != option->statements->end() )
	{
		// this is needed to manage correctly the statements assembly
		string statements_asm = "";

		// statements
		CoastStatementCollection::iterator it;
		for ( it = option->statements->begin() ; it != option->statements->end() ; it++ )
		{
			// We must reverse the assembly string.
			// See get_assembly( CoastApplication ).
			string tmp_asm = "    ";
			tmp_asm += get_assembly( *it );
			tmp_asm += "\n";
			statements_asm = tmp_asm + statements_asm;
		}

		assembly += statements_asm;
	}

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastSwitchDefaultOption*           option      )
{
	string assembly = "";

	assembly += "default:\n";

	// optional statements
	if ( option->statements->begin() != option->statements->end() )
	{
		// this is needed to manage correctly the statements assembly
		string statements_asm = "";

		// statements
		CoastStatementCollection::iterator it;
		for ( it = option->statements->begin() ; it != option->statements->end() ; it++ )
		{
			// We must reverse the assembly string.
			// See get_assembly( CoastApplication *& ).
			string tmp_asm = "    ";
			tmp_asm += get_assembly( *it );
			tmp_asm += "\n";
			statements_asm = tmp_asm + statements_asm;
		}

		assembly += statements_asm;
	}

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastSwitchStatement*               statement   )
{
	string assembly = "";

	assembly += "switch ( " + get_assembly( statement->expression ) + " )\n";
	assembly += "{\n";

	// optional cases
	if ( statement->case_options->begin() != statement->case_options->end() )
	{
		// this is needed to manage correctly the statements assembly
		string case_asm = "";

		// cases
		CoastSwitchCaseOptionCollection::iterator it;
		for ( it = statement->case_options->begin() ; it != statement->case_options->end() ; it++ )
		{
			// We must reverse the assembly string.
			// See get_assembly( CoastApplication *& ).
			string tmp_asm = "";
			tmp_asm += get_assembly( *it );
			tmp_asm += "\n";
			if ( statement->exclusive )
				tmp_asm += "    break;\n";
			case_asm = tmp_asm + case_asm;
		}

		assembly += case_asm;
	}

	// default
	if ( statement->default_option != NULL )
		assembly += get_assembly( statement->default_option ) + "\n";

	assembly += "}";

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastTypeReference*                 reference   )
{
	return reference->type;
}//get_assembly



string CCodeProvider::get_assembly( CoastUnaryOperatorExpression*       expression  )
{
	string assembly = "";

	assembly += "(";

	switch ( expression->op )
	{
		case Address:
			assembly += "&" + get_assembly( expression->value );
			break;
		case BitwiseNot:
			assembly += "~" + get_assembly( expression->value );
			break;
		case BooleanNot:
			assembly += "!" + get_assembly( expression->value );
			break;
		case Negative:
			assembly += "-" + get_assembly( expression->value );
			break;
		case Positive:
			assembly += "+" + get_assembly( expression->value );
			break;
		case PostDecrement:
			assembly += get_assembly( expression->value ) + "--";
			break;
		case PostIncrement:
			assembly += get_assembly( expression->value ) + "++";
			break;
		case PreDecrement:
			assembly += "--" + get_assembly( expression->value );
			break;
		case PreIncrement:
			assembly += "++" + get_assembly( expression->value );
			break;
		case Reference:
			assembly += "*" + get_assembly( expression->value );
			break;
		case SizeOf:
			assembly += "sizeof(" + get_assembly( expression->value ) + ")";
			break;
		default:
			assembly += "CoastUnaryOperatorType: Unmanaged Operator";
	}

	assembly += ")";

	return assembly;
}//get_assembly



string CCodeProvider::get_assembly( CoastVariableDeclarationStatement*  statement   )
{
	string assembly = "";
	
	assembly += get_assembly( statement->type ) + " " + statement->name;

	if ( statement->expression != NULL )
		assembly += " = " + get_assembly( statement->expression );

	return assembly + ";";
}//get_assembly



string CCodeProvider::get_assembly( CoastVariableReferenceExpression*   expression  )
{
	string assembly = expression->name;

	return assembly;
}//get_assembly

