
#ifndef _COAST_H_
#define _COAST_H_


#include <string>
#include <list>

using namespace std;



#define COAST_VERSION     "0.1.0"
#define COAST_DATE        "2003-02-12"
#define COAST_DESCRIPTION "First public release."
#define COAST_AUTHOR      "Gabriele Budelacci"





typedef enum
{
	BinaryOperator_NONE,		// C++ equivalence:
	Add,						//  +
	Assign,						//  =
	BitwiseAnd,					//  &
	BitwiseExor,				//  ^
	BitwiseOr,					//  |
	BooleanAnd,					//  &&
	BooleanOr,					//  ||
	Divide,						//  /
	GreaterThan,				//  >
	GreaterThanOrEqual,			//  >=
	IdentityEquality,			//  ==
	IdentityInequality,			//  !=
	LessThan,					//  <
	LessThanOrEqual,			//  <=
	Modulus,					//  %
	Multiply,					//  *
	ShiftLeft,					//  <<
	ShiftRight,					//  >>
	Subtract					//  -
//	ValueEquality
} CoastBinaryOperatorType;





typedef enum
{
	UnaryOperator_NONE,			// C++ equivalence:
	Address,					//  &
	BitwiseNot,					//  ~
	BooleanNot,					//  !
	Negative,					//  -
	Positive,					//  +
	PostDecrement,				//  --
	PostIncrement,				//  ++
	PreDecrement,				//  --
	PreIncrement,				//  ++
	Reference,					//  *
	SizeOf						//  sizeof
} CoastUnaryOperatorType;





/*
    CoastElement

    Abstract root class for coast code elements.
    (Gabriele Budelacci, 2003-01-22)
*/
class CoastElement
{
public:

	// polymorphic
	//	Needed to make this class polymorphic.
	virtual void polymorphic(){}

};//CoastElement





/*
    CoastExpression

    Abstract root class for coast code expressions.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastExpression : public CoastElement
{
};//CoastExpression





/*
    CoastArgumentReferenceExpression

    Class for argument reference access.
    (Gabriele Budelacci, 2003-02-05)
*/
class CoastArgumentReferenceExpression : public CoastExpression
{
public:

	// Constructor.
	CoastArgumentReferenceExpression( string name );

	// Destructor.
	virtual ~CoastArgumentReferenceExpression();


	// name:
	//	Argument name.
	string name;

};//CoastArgumentReferenceExpression





/*
    CoastOperatorExpression

    Abstract class for operators.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastOperatorExpression : public CoastExpression
{
};//CoastOperatorExpression





/*
    CoastBinaryOperatorExpression

    Class for binary operators.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastBinaryOperatorExpression : public CoastOperatorExpression
{
public:

	// Constructors.
	CoastBinaryOperatorExpression( CoastExpression* lvalue, CoastBinaryOperatorType op, CoastExpression* rvalue );

	// Destructor.
	virtual ~CoastBinaryOperatorExpression();


	// lvalue:
	//	Left value.
	CoastExpression* lvalue;

	// op:
	//	The operator.
	CoastBinaryOperatorType op;

	// rvalue:
	//	Right value.
	CoastExpression* rvalue;

};//CoastBinaryOperatorExpression





/*
    CoastUnaryOperatorExpression

    Class for unary operators.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastUnaryOperatorExpression : public CoastOperatorExpression
{
public:

	// Constructors.
	CoastUnaryOperatorExpression( CoastUnaryOperatorType op, CoastExpression* value );

	// Destructor.
	virtual ~CoastUnaryOperatorExpression();


	// op:
	//	The operator.
	CoastUnaryOperatorType op;

	// value:
	//	The value.
	CoastExpression* value;

};//CoastUnaryOperatorExpression





/*
    CoastPrimitiveExpression

    Class for primitive data type elements representation.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastPrimitiveExpression : public CoastExpression
{
public:

	// Constructors.
	CoastPrimitiveExpression();
	CoastPrimitiveExpression( bool   boolean          );
	CoastPrimitiveExpression( char   character        );
	CoastPrimitiveExpression( short  short_integer    );
	CoastPrimitiveExpression( int    integer          );
	CoastPrimitiveExpression( long   long_integer     );
	CoastPrimitiveExpression( float  single_precision );
	CoastPrimitiveExpression( double double_precision );
	CoastPrimitiveExpression( string str              );

	// Destructor.
	virtual ~CoastPrimitiveExpression();


	// type:
	//	Type of primitive element.
	int data_type;				// see follow

	// Primitive data.			// related type:
	bool   boolean;				// 1
	char   character;			// 2
	short  short_integer;		// 3
	int    integer;				// 4
	long   long_integer;		// 5
	float  single_precision;	// 6
	double double_precision;	// 7
	string strn;				// 8

};//CoastPrimitiveExpression





/*
    CoastVariableReferenceExpression

    Class for variable reference access.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastVariableReferenceExpression : public CoastExpression
{
public:

	// Constructor.
	CoastVariableReferenceExpression( string name );

	// Destructor.
	virtual ~CoastVariableReferenceExpression();


	// name:
	//	Variable name.
	string name;

};//CoastVariableReferenceExpression





/*
    CoastStatement

    Abstract root class for coast code statements.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastStatement : public CoastElement
{
};//CoastStatement





/*
    CoastAssignStatement

    Represent an assignment statement.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastAssignStatement : public CoastStatement
{
public:

	// Constructors:
	CoastAssignStatement( CoastExpression* lvalue, CoastExpression* rvalue );

	// Destructor:
	virtual ~CoastAssignStatement();


	// lvalue:
	//	Left expression value.
	CoastExpression* lvalue;

	// rvalue:
	//	Right expression value.
	CoastExpression* rvalue;

};//CoastAssignStatement





/*
    CoastBreakStatement

    Represent a break statement.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastBreakStatement : public CoastStatement
{
public:

	// Constructors:
	CoastBreakStatement();

	// Destructor:
	virtual ~CoastBreakStatement();

};//CoastBreakStatement





/*
    CoastCommentStatement

    Represent a comment.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastCommentStatement : public CoastStatement
{
public:

	// Constructors:
	CoastCommentStatement();
	CoastCommentStatement( string comment );

	// Destructor:
	virtual ~CoastCommentStatement();


	// comment:
	//	The comment.
	string comment;

};//CoastCommentStatement





/*
    CoastContinueStatement

    Represent a continue statement.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastContinueStatement : public CoastStatement
{
public:

	// Constructors:
	CoastContinueStatement();

	// Destructor:
	virtual ~CoastContinueStatement();

};//CoastContinueStatement





/*
    CoastExpressionStatement

    Class for a statement represented with an expression.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastExpressionStatement : public CoastStatement
{
public:

	// Constructors:
	CoastExpressionStatement( CoastExpression* expression );

	// Destructor:
	virtual ~CoastExpressionStatement();


	// expression:
	//	The expression.
	CoastExpression* expression;

};//CoastExpressionStatement





/*
    CoastTypeReference

    Represent a reference to a type.
    (Gabriele Budelacci, 2003-01-30)
*/
class CoastTypeReference : public CoastElement
{
public:

	// Constructor:
	CoastTypeReference( string type );

	// Destructor:
	virtual ~CoastTypeReference();


	// type:
	//	The type.
	string type;

};//CoastTypeReference





/*
    CoastArgumentDeclarationExpression

    Represent the declaration of a function argument.
    (Gabriele Budelacci, 2003-01-30)
*/
class CoastArgumentDeclarationExpression : public CoastExpression
{
public:

	// Constructor.
	CoastArgumentDeclarationExpression( CoastTypeReference* type, string name );

	// Destructor.
	virtual ~CoastArgumentDeclarationExpression();


	// type:
	//	Argument type.
	CoastTypeReference* type;

	// name:
	//	Argument name.
	string name;

};//CoastArgumentDeclarationExpression





/*
    CoastReturnStatement

    Represent the return statement.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastReturnStatement : public CoastStatement
{
public:

	// Constructors:
	CoastReturnStatement();
	CoastReturnStatement( CoastExpression* expression );

	// Destructor:
	virtual ~CoastReturnStatement();


	// expression:
	//	Optional return expression.
	CoastExpression* expression;

};//CoastReturnStatement





/*
    CoastVariableDeclarationStatement

    Represent the declaration of a variable.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastVariableDeclarationStatement : public CoastStatement
{
public:

	// Constructor.
	CoastVariableDeclarationStatement( CoastTypeReference* type, string name );
	CoastVariableDeclarationStatement( CoastTypeReference* type, string name, CoastExpression* expression );

	// Destructor.
	virtual ~CoastVariableDeclarationStatement();


	// type:
	//	Argument type.
	CoastTypeReference* type;

	// name:
	//	Argument name.
	string name;

	// expression:
	//	Init expression.
	CoastExpression* expression;

};//CoastVariableDeclarationStatement





/*
    CoastSnippetExpression

    Can represent any expression.
    (Gabriele Budelacci, 2003-01-28)
*/
class CoastSnippetExpression : public CoastExpression
{
public:

	// Constructor:
	CoastSnippetExpression( string expression );

	// Destructor:
	virtual ~CoastSnippetExpression();


	// expression:
	//	The expression.
	string expression;

};//CoastSnippetExpression





/*
    CoastSnippetStatement

    Can represent any statement.
    (Gabriele Budelacci, 2003-01-28)
*/
class CoastSnippetStatement : public CoastStatement
{
public:

	// Constructor:
	CoastSnippetStatement( string statement );

	// Destructor:
	virtual ~CoastSnippetStatement();


	// statement:
	//	The statement.
	string statement;

};//CoastSnippetStatement





/*
    CoastElementCollection

    Collection of coast code elements.
    (Gabriele Budelacci, 2003-01-24)
*/
class CoastElementCollection : public list<CoastElement*>
{
public:
	
	// Default constructor:
	CoastElementCollection();

	// Destructor:
	virtual ~CoastElementCollection();

	// add:
	//	Add an element to this collection.
	virtual void add( CoastElement* element );

};//CoastElementCollection





/*
    CoastExpressionCollection

    Collection of coast code expressions.
    (Gabriele Budelacci, 2003-01-30)
*/
class CoastExpressionCollection : public CoastElementCollection
{
public:

	// add:
	//	Add a expression to this collection.
	virtual void add( CoastExpression* expression );

protected:

	// add:
	//	Add an element to this collection.
	virtual void add( CoastElement* element );

};//CoastExpressionCollection





/*
    CoastArrayIndexerExpression

    Class for array indexer references.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastArrayIndexerExpression : public CoastExpression
{
public:

	// Constructors.
	CoastArrayIndexerExpression( CoastExpression* target, CoastExpression* index );

	// Destructor.
	virtual ~CoastArrayIndexerExpression();


	// target:
	//	Target object expression.
	CoastExpression* target;

	// indices:
	//	The index or indices of the indexer expression.
	CoastExpressionCollection* indices;

};//CoastArrayIndexerExpression





/*
    CoastFunctionInvokeExpression

    Represent a classic function call.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastFunctionInvokeExpression : public CoastExpression
{
public:

	// Constructor
	CoastFunctionInvokeExpression( string name );
	CoastFunctionInvokeExpression( string name, CoastExpression* argument );
	CoastFunctionInvokeExpression( string name, CoastExpressionCollection* arguments );

	// Destructor
	virtual ~CoastFunctionInvokeExpression();


	// name:
	//  Function name
	string name;

	// arguments:
	//	The arguments to be passed to this function.
	CoastExpressionCollection* arguments;

};//CoastFunctionInvokeExpression





/*
    CoastArgumentDeclarationExpressionCollection

    Collection of coast code statements.
    (Gabriele Budelacci, 2003-01-27)
*/
class CoastArgumentDeclarationExpressionCollection : public CoastExpressionCollection
{
public:

	// add:
	//	Add a argument declaration expression to this collection.
	virtual void add( CoastArgumentDeclarationExpression* expression );

protected:

	// add:
	//	Add an element to this collection.
	virtual void add( CoastExpression* expression );
	virtual void add( CoastElement* element );

};//CoastArgumentDeclarationExpressionCollection





/*
    CoastStatementCollection

    Collection of coast code statements.
    (Gabriele Budelacci, 2003-01-27)
*/
class CoastStatementCollection : public CoastElementCollection
{
public:

	// add:
	//	Add a statement to this collection.
	virtual void add( CoastStatement* statement );

protected:

	// add:
	//	Add an element to this collection.
	virtual void add( CoastElement* element );

};//CoastStatementCollection





/*
	CoastMember

	Root class for coast code members.
	(Gabriele Budelacci, 2003-02-08)
*/
class CoastMember : public CoastElement
{
};//CoastMember





/*
    CoastFunction

    This class represent a function.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastFunction : public CoastMember
{
public:

	// Constructor
	CoastFunction( string name );

	// Destructor
	virtual ~CoastFunction();


	// return_type:
	//	Return type for this function.
	CoastTypeReference* return_type;

	// name:
	//  Function name
	string name;

    // arguments:
    //	Collection of argument declarations.
    CoastArgumentDeclarationExpressionCollection* arguments;

    // statements:
    //	Collection of function statements.
    CoastStatementCollection* statements;

};//CoastFunction





/*
    CoastEntryPointFunction

    This class represent the entry point of an application.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastEntryPointFunction : public CoastFunction
{
public:

	// Default constructor
	CoastEntryPointFunction();

};//CoastEntryPointFunction





/*
    CoastSnippetMember

    Can represent any member.
    (Gabriele Budelacci, 2003-02-08)
*/
class CoastSnippetMember : public CoastMember
{
public:

	// Constructor:
	CoastSnippetMember( string member );

	// Destructor:
	virtual ~CoastSnippetMember();


	// statement:
	//	The statement.
	string member;

};//CoastSnippetMember





/*
    CoastIterationStatement

    Represent an iteration.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastIterationStatement : public CoastStatement
{
public:

	// Constructors:
	CoastIterationStatement();
	CoastIterationStatement( CoastStatement* init, CoastExpression* condition, CoastStatement* increment );
	CoastIterationStatement( CoastStatement* init, CoastExpression* condition, CoastStatement* increment, CoastStatementCollection* statements );

	// Destructor:
	virtual ~CoastIterationStatement();


	// init:
	//	The loop initialization statement.
	CoastStatement* init;

	// condition:
	//	Expression to test to exit condition.
	CoastExpression* condition;

	// increment:
	//	The per-cycle increment statement.
	CoastStatement* increment;

	// statements:
	//	The statements within the loop.
	CoastStatementCollection* statements;

};//CoastIterationStatement





/*
    CoastSwitchOption

    This abstract class represent a case or default option
    within a switch statement.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastSwitchOption : public CoastElement
{
};//CoastSwitchOption





/*
    CoastSwitchCaseOption

    This class represent a case option within a switch statement.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastSwitchCaseOption : public CoastSwitchOption
{
public:

	// Constructors:
	CoastSwitchCaseOption( CoastExpression* value );
	CoastSwitchCaseOption( CoastExpression* value, CoastStatementCollection* statements );

	// Destructor:
	virtual ~CoastSwitchCaseOption();


	// value:
	//	The value to be checked for this option.
	CoastExpression* value;

	// statement:
	//	The statements associated to this option.
	CoastStatementCollection* statements;

};//CoastSwitchCaseOption





/*
    CoastSwitchDefaultOption

    This class represent a default option within a switch statement.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastSwitchDefaultOption : public CoastSwitchOption
{
public:

	// Constructors:
	CoastSwitchDefaultOption();
	CoastSwitchDefaultOption( CoastStatementCollection* statements );

	// Destructor:
	virtual ~CoastSwitchDefaultOption();


	// statement:
	//	The statements associated to this option.
	CoastStatementCollection* statements;

};//CoastSwitchDefaultOption





/*
    CoastSwitchCaseOptionCollection

    Collection of coast case options.
    (Gabriele Budelacci, 2003-02-07)
*/
class CoastSwitchCaseOptionCollection : public CoastElementCollection
{
public:

	// add:
	//	Add a function to this collection.
	virtual void add( CoastSwitchCaseOption* option );

protected:

	// add:
	//	Add an element to this collection.
	virtual void add( CoastElement* element );

};//CoastSwitchCaseOptionCollection





/*
    CoastSwitchStatement

    Represent the switch statement.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastSwitchStatement : public CoastStatement
{
public:

	// Constructors:
	CoastSwitchStatement();
	CoastSwitchStatement( CoastExpression* expression );
	CoastSwitchStatement( CoastExpression* expression, bool exclusive );

	// Destructor:
	virtual ~CoastSwitchStatement();


	// exclusive:
	//	Determine if the options are mutually-exclusive.
	bool exclusive;

	// expression:
	//	The expression used to switch on.
	CoastExpression* expression;

	// case_options:
	//	The case options associated to this statement.
	CoastSwitchCaseOptionCollection* case_options;

	// default_option:
	//	The default option associated to this statement.
	CoastSwitchDefaultOption* default_option;

};//CoastSwitchStatement





/*
    CoastMemberCollection

    Collection of coast code members.
    (Gabriele Budelacci, 2003-01-24)
*/
class CoastMemberCollection : public CoastElementCollection
{
public:

	// add:
	//	Add a member to this collection.
	virtual void add( CoastMember* member );

protected:

	// add:
	//	Add an element to this collection.
	virtual void add( CoastElement* element );

};//CoastMemberCollection





/*
    CoastConditionStatement

    Represent a classic if statement.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastConditionStatement : public CoastStatement
{
public:

	// Constructors.
	CoastConditionStatement();
	CoastConditionStatement( CoastExpression* condition );

	// Destructor.
	virtual ~CoastConditionStatement();


	// condition:
	//	The condition.
	CoastExpression* condition;

    // true_statements:
    //	Container for true statements.
    CoastStatementCollection* true_statements;

    // false_statements:
    //	Container for false statements.
    CoastStatementCollection* false_statements;

};//CoastConditionStatement





/*
    CoastApplication

    Represent a generic application.
    (Gabriele Budelacci, 2003-01-23)
*/
class CoastApplication : public CoastElement
{
public:

    // Default constructor.
    CoastApplication();

    // Destructor.
    virtual ~CoastApplication();


    // functions:
    //	Collection of application functions.
    CoastMemberCollection* members;

};//CoastApplication





/*
    CoastProvider

    Abstract root class for coast providers.
    (Gabriele Budelacci, 2003-02-08)
*/
class CoastProvider
{
public:

	// compile_assembly:
	//	Compile assembly fom the specified source.
	virtual bool compile_assembly( CoastElement *ast ) = 0;

	// generate_code:
	//	Generate source code from the specified source.
	virtual string generate_code( CoastElement *ast ) = 0;

protected:

	// get_assembly:
	//	Get the assembly code of a element.
	virtual string get_assembly( CoastApplication*                   application ) = 0;
	virtual string get_assembly( CoastArgumentDeclarationExpression* expression  ) = 0;
	virtual string get_assembly( CoastArgumentReferenceExpression*   expression  ) = 0;
	virtual string get_assembly( CoastArrayIndexerExpression*        expression  ) = 0;
	virtual string get_assembly( CoastAssignStatement*               statement   ) = 0;
	virtual string get_assembly( CoastBinaryOperatorExpression*      expression  ) = 0;
	virtual string get_assembly( CoastBreakStatement*                statement   ) = 0;
	virtual string get_assembly( CoastCommentStatement*              statement   ) = 0;
	virtual string get_assembly( CoastConditionStatement*            statement   ) = 0;
	virtual string get_assembly( CoastContinueStatement*             statement   ) = 0;
	virtual string get_assembly( CoastEntryPointFunction*            function    ) = 0;
	virtual string get_assembly( CoastElement*                       element     ) = 0;
	virtual string get_assembly( CoastExpressionStatement*           statement   ) = 0;
	virtual string get_assembly( CoastFunction*                      function    ) = 0;
	virtual string get_assembly( CoastFunctionInvokeExpression*      expression  ) = 0;
	virtual string get_assembly( CoastIterationStatement*            statement   ) = 0;
	virtual string get_assembly( CoastPrimitiveExpression*           expression  ) = 0;
	virtual string get_assembly( CoastReturnStatement*               statement   ) = 0;
	virtual string get_assembly( CoastSnippetExpression*             expression  ) = 0;
	virtual string get_assembly( CoastSnippetMember*                 member      ) = 0;
	virtual string get_assembly( CoastSnippetStatement*              statement   ) = 0;
	virtual string get_assembly( CoastSwitchCaseOption*              option      ) = 0;
	virtual string get_assembly( CoastSwitchDefaultOption*           option      ) = 0;
	virtual string get_assembly( CoastSwitchStatement*               statement   ) = 0;
	virtual string get_assembly( CoastTypeReference*                 reference   ) = 0;
	virtual string get_assembly( CoastUnaryOperatorExpression*       expression  ) = 0;
	virtual string get_assembly( CoastVariableDeclarationStatement*  statement   ) = 0;
	virtual string get_assembly( CoastVariableReferenceExpression*   expression  ) = 0;

};//CoastProvider





#endif
